(*$ByteArray : MONO_ARRAY Byte List GeneralTypes InstreamType OutstreamType *)

require "MONO_ARRAY";
require "Byte";
require "List";
require "GeneralTypes";
require "InstreamType";
require "OutstreamType";

structure ByteArray: MONO_ARRAY =

(* ARRAYS OF BYTES

Created by:     Dave Berry, Harlequin Ltd.
                daveb@harlequin.co.uk
                daveb@harlequin.com

Date:           30 Sep 1993

Maintenance:    mlworks-support@harlequin.co.uk
                mlworks-support@harlequin.com


DESCRIPTION

   A ByteArray is a unique sequence of bytes that can be updated in
   place.  There is only one empty ByteArray.


RCS LOG

$Log: ByteArray.sml,v $
# Revision 1.1  1993/10/07  11:03:53  daveb
# Initial revision
#


*)

struct


(* LOCAL *)

  structure ByteArray = MLWorks.ByteArray

(* TYPES *)

  type Element = Byte.Byte

  type MonoArray = ByteArray.bytearray

  type T = MonoArray


(* CONVERTORS *)

  val list = ByteArray.to_list

  val fromList = ByteArray.from_list


(* CONSTANTS *)

  val empty = fromList []


(* CREATORS *)

  exception Size of string * int

  fun create size init =
	if size < 0 then raise Size ("create", size)
	else ByteArray.array (size, init)

  fun tabulate (size, f) =
	if size < 0 then raise Size ("tabulate", size)
        else ByteArray.tabulate (size, f)
		 
  fun tabulate' (size, f, base) =
	if size < 0 then raise Size ("tabulate'", size)
        else fromList (List.tabulate' (size, f, base))


(* ITERATORS *)

  val map = ByteArray.map

  val apply = ByteArray.iterate

  fun iterate f v = fromList (List.iterate f (list v))

  fun iterateApply f v = List.iterateApply f (list v)


(* OBSERVERS *)

  val size = ByteArray.length

  fun isEmpty v = (size v = 0)

  fun eq v v' = (v = v')

  fun ne v v' = (v <> v')


(* CONVERTORS *)

  fun stringSep start finish sep v =
        List.stringSep start finish sep Byte.string (list v)

  val string = ByteArray.to_string

  fun printSep start finish sep os v =
	List.printSep start finish sep Byte.print os (list v)

  fun print os v = printSep "" "" "" os v


(* SELECTORS *)

  exception Subscript of string * int

  fun sub (v, n) =
	ByteArray.sub (v, n)
	handle ByteArray.Subscript => raise Subscript ("sub", n)
  infix 9 sub;

  fun nth n v = v sub n
		handle Subscript _ => raise Subscript ("nth", n)

  exception Extract of int * int
  fun extract start finish v =
	ByteArray.subarray (v, start, finish - start)
	handle ByteArray.Subarray x => raise Extract x


(* MANIPULATORS *)

  val rev = ByteArray.rev

  infix 6 ^
  val op ^ = ByteArray.append

  exception Update of int
  fun update (v, i, x) =
	ByteArray.update (v, i, x)
	handle ByteArray.Subscript => raise Update i

  exception Copy of int * int * int
  fun copy v start finish v' start' =
        if finish < start orelse start < 0 orelse finish > size v orelse
           start' < 0 orelse start' + finish - start > size v'
        then raise Copy (start, finish, start')
	else ByteArray.copy (v, start, finish, v', start')

  exception UpdateRange of int * int
  fun updateRange v start finish i =
        if finish < start orelse start < 0 orelse finish > size v
        then raise UpdateRange (start, finish)
        else ByteArray.fill_range (v, start, finish, i)


(* REDUCERS *)

  exception Empty of string

  fun foldL f base v =
	ByteArray.reducel (fn (x, y) => f y x) (base, v)

  fun foldL' f v =
	if size v = 0 then raise Empty "foldL'"
	else List.foldL' f (list v)

  fun foldR f base v =
	ByteArray.reducer (General.uncurry f) (v, base)

  fun foldR' f v =
	if size v = 0 then raise Empty "foldR'"
	else List.foldR' f (list v)

  fun pairwise f v =
	List.pairwise f (list v)
end
