(*$System : SYSTEM General *)

require "loadSig";
require "SYSTEM";
require "General";
require "NonStandard";

loadSig "SYSTEM";

structure System: SYSTEM =

(* SYSTEM FUNCTIONS

Created by:     Dave Berry, Harlequin Ltd.
                daveb@harlequin.co.uk
                daveb@harlequin.com

Date:           30 Sep 1993

Maintenance:    mlworks-support@harlequin.co.uk
                mlworks-support@harlequin.com


DESCRIPTION

   MLWorks implements the cd, pwd, delete, quit, getenv and system functions.


NOTES

   This implementation of system always returns an empty string.


RCS LOG

$Log: System.sml,v $
# Revision 1.1  1993/10/07  11:03:53  daveb
# Initial revision
#


*)

struct


(* ML SYSTEM *)

  fun quit () = MLWorks.exit 0

  fun collect () = ();

  fun eq x y = false;

  fun hash _ = raise General.NotImplemented "hash";


(* INTERFACE TO OPERATING SYSTEM *)

  exception NoFile of string * string

  exception Permission of string * string

  val use = NonStandard.use

  val cd = MLWorks.OS.Unix.setwd

  fun isDir _ = raise General.NotImplemented "isDir"

  val pwd = MLWorks.OS.Unix.getwd

  fun dir _ = raise General.NotImplemented "dir"

  val delete = MLWorks.OS.Unix.unlink

  fun system s = (MLWorks.OS.system s; "")

  fun getenv key = 
    let 
      val keysize = (size key)
      fun searchkey nil = ""
        | searchkey (x::rest) =
          let val xsize = size x
          in
            if xsize > keysize andalso MLWorks.String.substring(x,0,keysize) = key then
              MLWorks.String.substring (x,keysize+1,xsize-(keysize+1))
          else
            searchkey rest
          end
    in
      searchkey (MLWorks.OS.Unix.environment ())
    end     

end
